#include "Database.h"

#include <Base\IO\BinaryWritingUtil.h>
#include <Base\IO\DynamicMemoryWriteStream.h>
#include <Base\IO\FileUtil.hpp>

#include <Base\String\Formatting.h>

namespace hyplex::drakan
{
	static const bool kEnableDump = true;

	ENodeType Sound::getType() const noexcept
	{
		return ENodeType::SOUND;
	}
	void Sound::dumpNode(const NodeDumpArg& arg) const
	{
		if (!kEnableDump)
			return;

		DynamicMemoryWriteStream bin;

		struct RIFF
		{
			uint8_t id[4] = { 'R', 'I', 'F', 'F' };
			uint32_t chunkSize = 0;
			uint8_t fmt[4] = { 'W', 'A', 'V', 'E' };
		} riff{};
		riff.chunkSize = 36 + (uint32_t)blob.size();
		write(bin, riff);

		struct Fmt
		{
			uint8_t id[4] = { 'f', 'm', 't', ' ' };
			uint32_t chunkSize = 16;
			uint16_t audioFormat = 1;
			uint16_t numChannels = 0;
			uint32_t sampleRate = 0;
			uint32_t bytesPerSecond = 0;
			uint16_t blockAlign = 0;
			uint16_t bitsPerSample = 0;
		} fmt{};
		fmt.numChannels = (uint16_t)numChannels;
		fmt.sampleRate = sampleRate;
		fmt.bytesPerSecond = sampleRate * numChannels * bitsPerSample / 8;
		fmt.blockAlign = (uint16_t)(numChannels * bitsPerSample / 8);
		fmt.bitsPerSample = (uint16_t)bitsPerSample;
		write(bin, fmt);

		writeArray(bin, "data"_sv);
		write(bin, uint32_t(blob.size()));
		writeArray(bin, blob);

		FileUnbufferedWriteStream(sprintfmt("%sSound%u %s.wav", arg.dir, id, escapeToFilename(name)), EFileWriteOpenMode::RECREATE, true)
			.write(asArrayView(bin.memory));
	}
}