#include <stdio.h>

#define WIN32_LEAN_AND_MEAN 1
#include <Windows.h>


int main(int argc, char* argv[])
{
	if (argc != 2)
	{
		fprintf(stderr, "Incorrect command line arguments. Expected just the path to dragon.rfl.\n");
		return 1;
	}

	const HMODULE hmodule = LoadLibraryA(argv[1]);
	if (!hmodule)
	{
		fprintf(stderr, "Failed to load RFL DLL.\n");
		return 1;
	}

	const auto dumpResource = [&](const char* name, const char* filename) {
		const HRSRC res = FindResourceA(hmodule, name, "ROC");
		if (!res)
		{
			fprintf(stderr, "Failed to find the \"ROC/%s\" resource.\n", name);
			return 1;
		}

		const HGLOBAL global = LoadResource(hmodule, res);

		const void* rawData = LockResource(global);
		const size_t rawSize = SizeofResource(hmodule, res);

		FILE* file = nullptr;
		fopen_s(&file, filename, "wb");
		if(!file || fwrite(rawData, rawSize, 1, file) != 1)
		{
			fprintf(stderr, "Failed to write file \"%s\".\n", filename);
			return 1;
		}
		fclose(file);

		return 0;
	};

	if (dumpResource("OBJECTS", "RFL Objects.bin") != 0)
		return 1;

	if (dumpResource("TYPES", "RFL Types.bin") != 0)
		return 1;

	return 0;
}